
# q-expansion of newform 336.6.a.v, downloaded from the LMFDB on 28 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 336
weight = 6
poly_data = [-48, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [9, 0], [-39, -5], [49, 0], [185, 37], [-528, 10], [-513, -3], [-140, 48], [-965, 163], [1278, 48], [-4704, -160], [-84, -922], [-4083, -845], [-13276, 112], [-16662, 202], [17952, 522], [9794, 2150], [-19514, -128], [-28846, -602], [-4779, -3299], [4356, -5350], [-16710, 1114], [-8876, -240], [-83859, 1395], [-23464, -3130], [-29809, -1359], [39908, 3340], [71177, 10277], [-87922, 4320], [-5170, 5148], [-167622, 3706], [172760, -12860], [-114140, -15302], [-68292, -12032], [-231072, -16374], [-360056, 10720], [154690, 16300], [39662, 11378], [177378, 18778], [65675, 30869], [-145273, -42773], [-57244, -7562], [343641, -12871], [578972, -16934], [846784, -1478], [-434556, 18540], [-751464, -17412], [-248252, -50924], [260074, -57418], [61268, -43946], [99628, -65566], [-747757, 16339], [465000, -22906], [1020514, -37994], [748293, 66307], [51971, -31573], [846153, -39085], [31348, -7340], [-259680, 152762], [755352, -61986], [628204, -4720], [830903, 88697], [1593984, 52828], [326642, 112330], [-1838518, 52864], [466624, -99998], [2228304, -5668], [215782, -56044], [-1778881, 39227], [-556586, 243808], [-269897, 130101], [244677, -269475], [-2764392, -8552], [-3158426, -114912], [963112, 12132], [-1842380, -275132], [299134, -115416], [-2273798, 86484], [1576192, 202606], [1369960, 142206], [5071154, 95278], [-858160, -125030], [-663263, 115361], [-3480530, -16116], [-2289672, 186144], [-494057, -275741], [62202, -296296], [6581194, 34080], [3556023, -60543], [-911762, 454830], [4016870, 245826], [-6012090, 261462], [-4566300, 279308], [-2648745, 222387], [891648, -165532], [-3429968, 70952], [-4339591, 202195], [-4849197, 454521], [-1747964, -254152], [-122448, -563774], [2601514, -512690], [-160052, 18014], [-277114, 126610], [4779964, 189218], [1356566, -38670], [1840382, -302564], [-2135058, -841710], [-11785489, -171011], [193713, -88119], [-963178, -841188], [-5650488, 499928], [2909950, 369496], [14016104, 281374], [-5534336, 879284], [6420562, -429158], [-11410180, 58682], [2616764, -959976], [5357742, -190778], [6646922, 360076], [16729365, -190071], [-10084438, -718260], [388272, -397906], [6818515, -290459], [-7755053, 363823], [3541072, 731084], [4129238, 248648], [12355326, -619464], [-14094384, -333136], [-3602436, 484180], [5468912, -934230], [13920758, 521754], [-9108443, 623517], [10693480, -1576], [-176322, 1467752], [-13909803, 29067], [12514374, -1161932], [-22419901, 282405], [-21374888, 426036], [17520891, -765099], [-18602022, -441872], [-1452792, 604348], [12211060, 17150], [5124710, -942610], [-518591, -1180947], [15769052, -519498], [-33376434, 177606], [2110574, 1887648], [-1464595, 2581667], [-24246672, 61316], [24480039, -409433], [38996390, 93944], [-21539877, 1269417], [15572672, 390372], [-16753774, 735610], [-2276588, 1200144], [15908601, -430407], [1209988, -148692], [1340033, 2074351], [10501078, 919884], [-8336791, 296195], [-3143903, 1091765], [-32981342, 762088], [39768722, -485318], [8713556, -3227608], [2986248, 1949302], [21607876, 339628], [38216300, 1174204], [-3697286, 2390604], [-34084352, 235982], [23774391, 175641], [-35267075, 289897], [-7423630, 2472124], [23371073, -834775], [-23555096, -1854274], [-11252404, 2206044], [25380575, -959995], [16380138, -2425802], [-36550762, 1769152], [22917960, -492000], [-1790102, 1319924], [-1958610, 2044270], [-21517896, 378516], [-43653432, 1243234], [10423883, -91903], [-46809393, -1800433], [22628515, 2835781], [-5621374, -3585964], [-38871308, 2796128], [36210288, -1962490], [55143892, -837372], [10839656, 2955462], [-29505911, -454555], [-58971844, 346848], [31816783, -1105079], [1314773, -1641271], [24412599, -2022771], [-18243864, 869686], [9695832, 3934170], [-25106991, 4533199], [32726158, -1440554], [70444576, -952014], [7659732, -325372], [19565376, 1956738], [71027240, 958982], [-34536728, -3501060], [9703619, -3134675], [-24183444, 2228556], [-10888231, 1211869], [103226882, 589976], [-21991816, 3039068], [20479578, -1424984], [-46748203, -4424473], [-20873396, 3534852], [42661724, -95872], [56422826, 4398002], [15571174, 943628], [-9192236, -7426412], [-48795925, 2836457], [52044963, -1582245], [88830822, 87128], [-40714714, -2146368], [-5169084, 1604524], [-93955818, 195188], [-13090448, 9590464], [-62466176, -863236], [-98183428, 1174598], [-18455107, 684035], [15986731, 5050491], [-43594292, -4349116], [91805515, 3311351], [-14592434, 3435584], [-22854416, -2499716], [-27901924, 6899596], [-110774926, -3172856], [48030872, -5511252], [29102648, 9339592], [-5944960, 6576414], [-58586828, 7828190], [-9095197, -1762033], [44978760, 8131280], [38204219, -2859569], [110590200, 369404], [-39571892, 3613956], [-109100104, 3690234], [-97985127, 2249223], [-2598630, -1421118], [-32129040, -2208160], [-31892390, -4845850], [-19426866, 10798810], [-64411505, -1642721], [-11272878, -10375996], [-141446385, -2388051], [-91158239, 1687433], [-65549780, 8127554], [13647427, -3084515], [-21509595, -9150615], [41443712, 4299370], [11311336, 7231260], [-13894885, 9593277], [77217406, -4119308], [23372378, 9578586], [-92688141, 1667031], [-83568148, -416546], [-14486146, -9142056], [20788411, -14796663], [-209592, 11362372], [16618484, 3059622], [-155101397, 1295633], [15052898, -9051346], [20457326, -8155464], [92145230, 6360616], [177780476, 2945936], [29034996, -5649254], [18793468, 4791164], [107594092, 1883002], [65331780, -9966100], [63702333, 5782649], [29608464, -2720638], [-6917860, 10037554], [133900346, -6504490], [7068532, 964276], [-21373102, -8185302], [-61816652, -3149956], [-21875342, 6283940], [-20667820, -18361048], [-19989167, -1332783], [-32696334, -1245920], [-138385180, 1943470], [-94034324, -6169980], [-201501571, -1728965], [-62558168, -4463654], [11461288, -7904924], [159637304, -6223970], [-122506010, 7032346], [-138300314, 4449672], [-63587653, 11912725], [27760212, 436036], [-295921259, -622409], [-85842934, -13429826], [-162168060, -10746496], [14982834, 15078744], [119442430, -5327520], [12977142, -8237866], [110306517, 89481], [76296870, -2435710], [60574654, -17877252], [103027219, -180385], [-2812236, -21154690], [-83223987, 6382005], [34206984, 16899922], [80872428, 9965436], [-264824830, 3782908], [-97035096, 18362182], [23836648, -6555508], [103613963, -3598285], [182200164, 5002778], [-120907928, 5773652], [71036235, 13147947], [-49992004, -1111814], [-61968796, 13942866], [111153756, -363544], [172621960, 12210414], [-3795521, -5758263], [12092756, -1363836], [-118962442, 1298188], [229070260, -8900526], [24888954, -12087874], [-260976660, -6666352], [260816453, -4802539], [-293148194, 6747560], [27581578, -8756116], [-24402598, 21088300], [-356403140, -5807812], [-46462158, -8340274], [-286693062, -8117834], [-77251278, 11638], [-192515482, 20844392], [53332357, -4017533], [90600656, -10256666], [923604, 4227428], [-104295418, -3932256], [329048664, -178242], [-149424171, -1528857], [-222370772, -2615452], [277309494, -12486520], [-74752643, -30479935], [-155336664, -5450998], [-210088292, 14587624], [-147779692, -6350108], [312413159, 15433753], [-509287448, 2365924], [191998104, -5249682], [-33631960, 19002778], [180117084, -19213428], [-16098474, -1938224], [-62513329, 16012597], [290213056, 17123696], [-219414772, 1075616], [147023766, 13734468], [20024009, -19500879], [24116910, 18120408], [-121583279, 14150567], [185477257, -18008215], [-175036499, 2391033], [363045200, 4606716], [-338077840, 3409958], [-80214419, -594089], [145730426, -384702], [267325212, -19631838], [78689475, -7874217], [-280278392, 3767436], [182001573, -15515915], [-394495404, -12728450], [18472372, 32906268], [-107074336, -23909742], [-59154224, -171956], [114052579, -23238317], [-50320962, 34477980], [30330167, 919157], [-151620020, 7080834], [-27794361, 24177153], [457275762, -12213912], [371076508, -13118012], [18613518, -17319636], [-149029980, 21031584], [-106992916, 7099220], [-435607480, 2042568], [-231335224, 241610], [109815042, -37761362], [355367014, -2533834], [-30466080, 8193310], [158576425, -22099189], [-296063583, 27617373], [277119048, -20371660], [-260031715, -25325019], [-335463298, -23924764], [-128530892, -19799532], [22534823, 28893229], [-297517134, -31179602], [67179994, 20230388], [156034534, 27435360], [519080478, 7449084], [-198837118, -37761982], [-206239507, -39911645], [161609529, -2442869], [-188632888, -37751472], [-366201936, -31545198], [316490146, -26818160], [267534980, 37962576], [-16148502, 20726790], [-592906038, 2229688], [118431102, -35157288], [-228703669, -3456217], [-66356252, 4840216], [410054040, -20309042], [196003261, 15861959], [159257815, 44989271], [114507984, 1534360], [424648755, -1845231], [-96085566, -44142246], [-238991746, 29523728], [-260267710, -10119404], [126672281, -49742935], [164996304, -39028508], [-430166374, 5192192], [-2241917, 28922621], [-194406853, -14434961], [425352504, -36336642], [-49169988, -19479696], [-374823310, -11822566]]
